@@ -119,7 +119,7 @@ def login():
     if not verify_password(username, password):
         logging.warning(f"LOGIN FAILED: {username} {password}")
         return make_response(jsonify({
-            'error': "wrong user or password"
+            'error': "Login failed: wrong user or password"
         }), 401)
     else:
         user = curr_app.users.get(username)
@@ -408,7 +408,12 @@ def get_document_elements(workspace_id, document_uri):
     :param workspace_id:
     :param document_uri:
     :request_arg category_id:
+    :request_arg size: number of elements to return
+    :request_arg start_idx: get elements starting from this index (for pagination)
     """
+
+    size = int(request.args.get('size', curr_app.config["CONFIGURATION"].main_panel_elements_per_page))
+    start_idx = int(request.args.get('start_idx', 0))
     dataset_name = curr_app.orchestrator_api.get_dataset_name(workspace_id)
     document = curr_app.orchestrator_api.get_documents(workspace_id, dataset_name, [document_uri])[0]
     elements = document.text_elements
@@ -416,8 +421,9 @@ def get_document_elements(workspace_id, document_uri):
     if category_id is not None:
         category_id = int(category_id)
     elements_transformed = elements_back_to_front(workspace_id, elements, category_id)
+    elements_transformed = elements_transformed[start_idx: start_idx + size]
 
-    res = {'elements': elements_transformed}
+    res = {'elements': elements_transformed, 'hit_count': len(elements)}
     return jsonify(res)
 
 
@@ -437,7 +443,7 @@ def get_document_positive_predictions(workspace_id, document_uri):
     :request_arg start_idx: get elements starting from this index (for pagination)
     """
     category_id = int(request.args['category_id'])
-    size = int(request.args.get('size', 100))
+    size = int(request.args.get('size', curr_app.config["CONFIGURATION"].sidebar_panel_elements_per_page))
     start_idx = int(request.args.get('start_idx', 0))
 
     if len(curr_app.orchestrator_api.get_all_iterations_by_status(workspace_id, category_id,
@@ -474,7 +480,7 @@ def get_positive_predictions(workspace_id):
     :request_arg start_idx: get elements starting from this index (for pagination)
     """
     category_id = int(request.args['category_id'])
-    size = int(request.args.get('size', 100))
+    size = int(request.args.get('size', curr_app.config["CONFIGURATION"].sidebar_panel_elements_per_page))
     start_idx = int(request.args.get('start_idx', 0))
 
     all_ready_iterations = curr_app.orchestrator_api.get_all_iterations_by_status(workspace_id, category_id,
@@ -488,14 +494,14 @@ def get_positive_predictions(workspace_id):
         positive_predicted_elements = curr_app.orchestrator_api.sample_elements_by_prediction(
             workspace_id, category_id, unlabeled_only=False, required_label=LABEL_POSITIVE,
             remove_duplicates=remove_duplicates)
-        iteration, iteration_idx = all_ready_iterations[-1]
+        iteration, _ = all_ready_iterations[-1]
 
         sorted_elements = sorted(positive_predicted_elements, key=lambda te: get_natural_sort_key(te.uri))
         elements_transformed = elements_back_to_front(workspace_id, sorted_elements, category_id)
         elements_transformed = elements_transformed[start_idx: start_idx + size]
 
         res = {'hit_count': len(positive_predicted_elements), "positive_fraction":
-            iteration.iteration_statistics["positive_fraction"], 'elements': elements_transformed}
+               iteration.iteration_statistics["positive_fraction"], 'elements': elements_transformed}
         return jsonify(res)
 
 
@@ -525,27 +531,27 @@ def query(workspace_id):
 
     :param workspace_id:
     :request_arg category_id:
-    :request_arg qry_string: regular expression
-    :request_arg qry_size: number of elements to return
-    :request_arg sample_start_idx: get elements starting from this index (for pagination)
+    :request_arg qry_string: query string
+    :request_arg size: number of elements to return
+    :request_arg start_idx: get elements starting from this index (for pagination)
     """
     category_id = request.args.get('category_id')
     if category_id is not None:
         category_id = int(category_id)
     query_string = request.args.get('qry_string')
-    sample_size = int(request.args.get('qry_size', 100))
-    sample_start_idx = int(request.args.get('sample_start_idx', 0))
+    sample_size = int(request.args.get('size', curr_app.config["CONFIGURATION"].sidebar_panel_elements_per_page))
+    sample_start_idx = int(request.args.get('start_idx', 0))
 
     dataset_name = curr_app.orchestrator_api.get_dataset_name(workspace_id)
-    resp = curr_app.orchestrator_api.query(workspace_id, dataset_name, category_id=None, query_regex=query_string,
+    resp = curr_app.orchestrator_api.query(workspace_id, dataset_name, category_id=None,
+                                           query=query_string, is_regex=False,
                                            unlabeled_only=False, sample_size=sample_size,
                                            sample_start_idx=sample_start_idx, remove_duplicates=True)
-
     sorted_elements = sorted(resp["results"], key=lambda te: get_natural_sort_key(te.uri))
     elements_transformed = elements_back_to_front(workspace_id, sorted_elements, category_id)
-
     res = {'elements': elements_transformed,
-           'hit_count': resp["hit_count"], 'hit_count_unique': resp["hit_count_unique"]}
+           'hit_count': resp["hit_count"],
+           'hit_count_unique': resp["hit_count_unique"]}
     return jsonify(res)
 
 
@@ -614,11 +620,16 @@ def get_all_positively_labeled_elements_for_category(workspace_id):
 
     :param workspace_id:
     :request_arg category_id:
+    :request_arg size: number of elements to return
+    :request_arg start_idx: get elements starting from this index (for pagination)
     """
+    size = int(request.args.get('size', curr_app.config["CONFIGURATION"].sidebar_panel_elements_per_page))
+    start_idx = int(request.args.get('start_idx', 0))
     category_id = int(request.args['category_id'])
-    elements_transformed = get_all_labeled_elements(workspace_id, category_id, label=LABEL_POSITIVE)
+    elements, hit_count = get_all_labeled_elements(workspace_id, category_id, label=LABEL_POSITIVE, size=size,
+                                                   start_idx=start_idx)
 
-    res = {'positive_elements': elements_transformed}
+    res = {'elements': elements, "hit_count": hit_count}
     return jsonify(res)
 
 
@@ -632,31 +643,40 @@ def get_all_negatively_labeled_elements_for_category(workspace_id):
 
     :param workspace_id:
     :request_arg category_id:
+    :request_arg size: number of elements to return
+    :request_arg start_idx: get elements starting from this index (for pagination)
     """
     category_id = int(request.args['category_id'])
-    elements_transformed = get_all_labeled_elements(workspace_id, category_id, label=LABEL_NEGATIVE)
+    size = int(request.args.get('size', curr_app.config["CONFIGURATION"].sidebar_panel_elements_per_page))
+    start_idx = int(request.args.get('start_idx', 0))
+   
+    elements, hit_count = get_all_labeled_elements(workspace_id, category_id, label=LABEL_NEGATIVE, size=size,
+                                                   start_idx=start_idx)
 
-    res = {'negative_elements': elements_transformed}
+    res = {'elements': elements, "hit_count": hit_count}
     return jsonify(res)
 
 
-def get_all_labeled_elements(workspace_id, category_id, label=None):
+def get_all_labeled_elements(workspace_id, category_id, label=None, size: int = 100, start_idx: int = 0):
     """
     Return all elements that were assigned the given label, or all labeled elements if label is None
 
     :param workspace_id:
     :param category_id:
-    :param label:
+    :param label: return only elements assigned this label; if None, return all labeled elements
+    :param size: number of elements to return
+    :param start_idx: get elements starting from this index (for pagination)
     """
     dataset_name = curr_app.orchestrator_api.get_dataset_name(workspace_id)
     elements = curr_app.orchestrator_api.get_all_labeled_text_elements(workspace_id, dataset_name, category_id,
                                                                        remove_duplicates=True)
-
     if label is not None:
         elements = [element for element in elements
                     if element.category_to_label[category_id].label == label]
     elements_transformed = elements_back_to_front(workspace_id, elements, category_id)
-    return elements_transformed
+    hit_count = len(elements_transformed)
+    elements_transformed = elements_transformed[start_idx: start_idx + size]
+    return elements_transformed, hit_count
 
 
 @main_blueprint.route('/workspace/<workspace_id>/import_labels', methods=['POST'])
@@ -691,8 +711,13 @@ def export_labels(workspace_id):
     for a specific category. Column names for the various fields are listed under DisplayFields.
 
     :param workspace_id:
+    :request_arg labeled_only: only export elements as they were labeled by the user. If set to False, use the
+    TrainingSetSelectionStrategy to determine the exported elements
     """
-    return curr_app.orchestrator_api.export_workspace_labels(workspace_id).to_csv(index=False)
+    labeled_only = request.args.get('labeled_only', "true")
+    labeled_only = labeled_only.lower() == "true"
+
+    return curr_app.orchestrator_api.export_workspace_labels(workspace_id, labeled_only).to_csv(index=False)
 
 
 """
@@ -768,17 +793,17 @@ def get_elements_to_label(workspace_id):
     :request_arg start_idx: get elements starting from this index (for pagination)
     """
     category_id = int(request.args['category_id'])
-    size = int(request.args.get('size', 100))
+    size = int(request.args.get('size', curr_app.config["CONFIGURATION"].sidebar_panel_elements_per_page))
     start_idx = int(request.args.get('start_idx', 0))
 
     if len(curr_app.orchestrator_api.get_all_iterations_by_status(workspace_id, category_id,
                                                                   IterationStatus.READY)) == 0:
         return jsonify({"elements": []})
 
-    elements = curr_app.orchestrator_api.get_elements_to_label(workspace_id, category_id, size, start_idx)
+    elements, hit_count = curr_app.orchestrator_api.get_elements_to_label(workspace_id, category_id, size, start_idx)
     elements_transformed = elements_back_to_front(workspace_id, elements, category_id)
 
-    res = {'elements': elements_transformed}
+    res = {'elements': elements_transformed, 'hit_count': hit_count}
     return jsonify(res)
 
 
@@ -931,17 +956,23 @@ def get_suspicious_elements(workspace_id):
 
     :param workspace_id:
     :request_arg category_id:
+    :request_arg size: number of elements to return
+    :request_arg start_idx: get elements starting from this index (for pagination)
     """
-
+    size = int(request.args.get('size', curr_app.config["CONFIGURATION"].sidebar_panel_elements_per_page))
+    start_idx = int(request.args.get('start_idx', 0))
     category_id = int(request.args['category_id'])
+    
     try:
         suspicious_elements = curr_app.orchestrator_api.get_suspicious_elements_report(workspace_id, category_id)
         elements_transformed = elements_back_to_front(workspace_id, suspicious_elements, category_id)
-        res = {'elements': elements_transformed}
+        hit_count = len(elements_transformed)
+        elements_transformed = elements_transformed[start_idx: start_idx + size]
+        res = {'elements': elements_transformed, 'hit_count': hit_count}
         return jsonify(res)
     except Exception:
         logging.exception("Failed to generate suspicious elements report")
-        res = {'elements': []}
+        res = {'elements': [], 'hit_count': 0}
         return jsonify(res)
 
 
@@ -962,15 +993,21 @@ def get_contradicting_elements(workspace_id):
 
     :param workspace_id:
     :request_arg category_id:
+    :request_arg size: number of elements to return
+    :request_arg start_idx: get elements starting from this index (for pagination)
     """
     category_id = int(request.args['category_id'])
+    size = int(request.args.get('size', curr_app.config["CONFIGURATION"].sidebar_panel_elements_per_page))
+    start_idx = int(request.args.get('start_idx', 0))
     try:
         contradiction_elements_dict = curr_app.orchestrator_api.get_contradiction_report(workspace_id, category_id)
         element_pairs_transformed = [elements_back_to_front(workspace_id, element_pair, category_id)
                                      for element_pair in contradiction_elements_dict['pairs']]
+        hit_count = len(element_pairs_transformed)
+        element_pairs_transformed = element_pairs_transformed[start_idx: start_idx + size]
         diffs = [[list(element_a_unique_token_set), list(element_b_unique_token_set)]
                  for element_a_unique_token_set, element_b_unique_token_set in contradiction_elements_dict['diffs']]
-        res = {'pairs': element_pairs_transformed, 'diffs': diffs}
+        res = {'pairs': element_pairs_transformed, 'diffs': diffs, 'hit_count': hit_count}
         return jsonify(res)
     except Exception:
         logging.exception(f"Failed to generate contradiction report for workspace "
@@ -1139,7 +1176,9 @@ def get_feature_flags():
     """
 
     res =  {
-        "login_required": curr_app.config['CONFIGURATION'].login_required
+        "login_required": curr_app.config['CONFIGURATION'].login_required,
+        "main_panel_elements_per_page": curr_app.config['CONFIGURATION'].main_panel_elements_per_page,
+        "sidebar_panel_elements_per_page": curr_app.config['CONFIGURATION'].sidebar_panel_elements_per_page
     }
     logging.debug(f'Feature flags are: {res}')
     return jsonify(res) 
