@@ -31,7 +31,7 @@ from typing import Sequence, Iterable, Mapping, List, Union
 import label_sleuth.data_access.file_based.utils as utils
 from label_sleuth.data_access.core.data_structs import Document, Label, TextElement
 from label_sleuth.data_access.data_access_api import DataAccessApi, AlreadyExistsException, DocumentStatistics, \
-    LabeledStatus, get_document_uri
+    LabeledStatus
 
 
 class FileBasedDataAccess(DataAccessApi):
@@ -79,8 +79,9 @@ class FileBasedDataAccess(DataAccessApi):
             doc_ids = {document.uri for document in documents}
             intersection = doc_ids.intersection(set(self.get_all_document_uris(dataset_name)))
             if len(intersection) > 0:
-                raise AlreadyExistsException(f"{len(intersection)} documents are already in dataset '{dataset_name}'."
-                                             f" uris: ({intersection})", list(intersection))
+                raise AlreadyExistsException(f"{len(intersection)} documents are already in dataset '{dataset_name}'. "
+                                             f"uris: ({list(intersection)[:5]}{'...' if len(intersection)>5 else ''})",
+                                             list(intersection))
 
         for doc in documents:
             # save doc to file
@@ -225,7 +226,7 @@ class FileBasedDataAccess(DataAccessApi):
         return utils.build_text_elements_from_dataframe_and_labels(self._get_ds_in_memory(dataset_name), labels_dict={})
 
     def get_text_elements(self, workspace_id: str, dataset_name: str, sample_size: int = sys.maxsize,
-                          sample_start_idx: int = 0, query_regex: str = None, document_uri=None,
+                          sample_start_idx: int = 0, query: str = None, is_regex: bool = False, document_uri=None,
                           remove_duplicates=False, random_state: int = 0) -> Mapping:
         """
         Sample *sample_size* TextElements from dataset_name, optionally limiting to those matching a query,
@@ -235,8 +236,9 @@ class FileBasedDataAccess(DataAccessApi):
         :param dataset_name: the name of the dataset from which TextElements are sampled
         :param sample_size: how many TextElements should be sampled
         :param sample_start_idx: get elements starting from this index (for pagination). Default is 0
-        :param query_regex: a regular expression that should be matched in the sampled TextElements. If None, then
-        no such filtering is performed.
+        :param query: a query string to search for in the sampled TextElements.
+                      If None, then no such filtering is performed.
+        :param is_regex: if True, the query string is interpreted as a regular expression (False by default)
         :param document_uri: get elements from a particular document
         :param remove_duplicates: if True, do not include elements that are duplicates of each other.
         :param random_state: provide an int seed to define a random state. Default is zero.
@@ -248,14 +250,15 @@ class FileBasedDataAccess(DataAccessApi):
             results_dict = \
                 self._get_text_elements(
                     workspace_id=workspace_id, dataset_name=dataset_name,
-                    filter_func=lambda df, _: utils.filter_by_query_and_document_uri(df, query_regex, document_uri),
+                    filter_func=lambda df, _: utils.filter_by_query_and_document_uri(df, query, is_regex, document_uri),
                     sample_size=sample_size, sample_start_idx=sample_start_idx,
                     remove_duplicates=remove_duplicates, random_state=random_state)
 
         return results_dict
 
     def get_unlabeled_text_elements(self, workspace_id: str, dataset_name: str, category_id: int,
-                                    sample_size: int = sys.maxsize, sample_start_idx: int = 0, query_regex: str = None,
+                                    sample_size: int = sys.maxsize, sample_start_idx: int = 0,
+                                    query: str = None, is_regex: bool = False,
                                     remove_duplicates=False, random_state: int = 0) -> Mapping:
         """
         Sample *sample_size* TextElements from dataset_name, unlabeled for category_id in workspace_id, optionally
@@ -266,8 +269,9 @@ class FileBasedDataAccess(DataAccessApi):
         :param category_id: we demand that the elements are not labeled for this category
         :param sample_size: how many TextElements should be sampled
         :param sample_start_idx: get elements starting from this index (for pagination). Default is 0
-        :param query_regex: a regular expression that should be matched in the sampled TextElements. If None, then
-        no such filtering is performed.
+        :param query: a query string to search for in the sampled TextElements.
+                      If None, then no such filtering is performed.
+        :param is_regex: if True, the query string is interpreted as a regular expression (False by default)
         :param remove_duplicates: if True, do not include elements that are duplicates of each other.
         :param random_state: provide an int seed to define a random state. Default is zero.
         :return: a dictionary with two keys: 'results' whose value is a list of TextElements, and 'hit_count' whose
@@ -275,7 +279,7 @@ class FileBasedDataAccess(DataAccessApi):
         {'results': [TextElement], 'hit_count': int}
         """
         filter_func = lambda df, labels: \
-            utils.filter_by_query_and_label_status(df, labels, category_id, LabeledStatus.UNLABELED, query_regex)
+            utils.filter_by_query_and_label_status(df, labels, category_id, LabeledStatus.UNLABELED, query, is_regex)
 
         with self._get_lock_object_for_workspace(workspace_id):
             results_dict = self._get_text_elements(workspace_id=workspace_id, dataset_name=dataset_name,
@@ -285,7 +289,7 @@ class FileBasedDataAccess(DataAccessApi):
         return results_dict
 
     def get_labeled_text_elements(self, workspace_id: str, dataset_name: str, category_id: int,
-                                  sample_size: int = sys.maxsize, query_regex: str = None,
+                                  sample_size: int = sys.maxsize, query: str = None, is_regex: bool = False,
                                   remove_duplicates=False, random_state: int = 0) -> Mapping:
         """
         Sample *sample_size* TextElements from dataset_name, labeled for category_id in workspace_id,
@@ -295,8 +299,9 @@ class FileBasedDataAccess(DataAccessApi):
         :param dataset_name: the name of the dataset from which TextElements are sampled
         :param category_id: we demand that the elements are labeled for this category
         :param sample_size: how many TextElements should be sampled
-        :param query_regex: a regular expression that should be matched in the sampled TextElements. If None, then
-        no such filtering is performed.
+        :param query: a query string to search for in the sampled TextElements.
+                      If None, then no such filtering is performed.
+        :param is_regex: if True, the query string is interpreted as a regular expression (False by default)
         :param remove_duplicates: if True, do not include elements that are duplicates of each other.
         :param random_state: provide an int seed to define a random state. Default is zero.
         :return: a dictionary with two keys: 'results' whose value is a list of TextElements, and 'hit_count' whose
@@ -304,8 +309,7 @@ class FileBasedDataAccess(DataAccessApi):
         {'results': [TextElement], 'hit_count': int}
         """
         filter_func = lambda df, labels: \
-            utils.filter_by_query_and_label_status(df, labels, category_id, LabeledStatus.LABELED, query_regex)
-
+            utils.filter_by_query_and_label_status(df, labels, category_id, LabeledStatus.LABELED, query, is_regex)
         with self._get_lock_object_for_workspace(workspace_id):
             results_dict = self._get_text_elements(workspace_id=workspace_id, dataset_name=dataset_name,
                                                    filter_func=filter_func, sample_size=sample_size,
@@ -323,17 +327,16 @@ class FileBasedDataAccess(DataAccessApi):
         :return: a map whose keys are label values, and the values are the number of TextElements this label was
         assigned to.
         """
-        labels_by_uri = self._get_labels(workspace_id=workspace_id, dataset_name=dataset_name)
+        labels_by_uri = self._get_labels(workspace_id=workspace_id, dataset_name=dataset_name).copy()
         if remove_duplicates:
             corpus_df = self._get_ds_in_memory(dataset_name=dataset_name)
             labeled_df = corpus_df[corpus_df['uri'].isin(labels_by_uri)]
-            unique_texts = set()
-            uris_to_keep = {t.uri for t in labeled_df.itertuples()
-                            if t.text not in unique_texts and not unique_texts.add(t.text)}
-            labels_by_uri = {key: value for key, value in labels_by_uri.items() if key in uris_to_keep}
+            uris_to_keep = set(labeled_df.drop_duplicates(subset=['text'])['uri'])
+            labels_by_uri = {uri: category_to_label for uri, category_to_label in labels_by_uri.items()
+                             if uri in uris_to_keep}
         category_label_list = \
-            [labels_by_uri[uri][category_id].label for uri in labels_by_uri
-             if category_id in labels_by_uri[uri]]
+            [category_to_label[category_id].label for category_to_label in labels_by_uri.values()
+             if category_id in category_to_label]
         category_label_counts = Counter(category_label_list)
         return category_label_counts
 
