@@ -218,16 +218,18 @@ class OrchestratorApi:
                                                             sample_size=sys.maxsize,
                                                             remove_duplicates=remove_duplicates)['results']
 
-    def query(self, workspace_id: str, dataset_name: str, category_id: Union[int, None], query_regex: str,
+    def query(self, workspace_id: str, dataset_name: str, category_id: Union[int, None],
+              query: str, is_regex: bool = False,
               sample_size: int = sys.maxsize, sample_start_idx: int = 0, unlabeled_only: bool = False,
-              remove_duplicates=False) -> Mapping[str, object]:
+              remove_duplicates=False) -> Mapping[str, Union[List[TextElement], int]]:
         """
         Query a dataset using the given regex, returning up to *sample_size* elements that meet the query
     
         :param workspace_id:
         :param dataset_name:
         :param category_id: optional. If unlabeled_only is True category_id will be used for determining unlabeled
-        :param query_regex: string
+        :param query: string
+        :param is_regex: if True, the query string is interpreted as a regular expression (False by default)
         :param unlabeled_only: if True, filters out labeled elements
         :param sample_size: maximum items to return
         :param sample_start_idx: get elements starting from this index (for pagination)
@@ -247,7 +249,7 @@ class OrchestratorApi:
         else:
             return self.data_access.get_text_elements(workspace_id=workspace_id, dataset_name=dataset_name,
                                                       sample_size=sample_size, sample_start_idx=sample_start_idx,
-                                                      query_regex=query_regex, remove_duplicates=remove_duplicates)
+                                                      query=query, is_regex=is_regex, remove_duplicates=remove_duplicates)
 
     def set_labels(self, workspace_id: str, uri_to_label: Mapping[str, Mapping[int, Label]],
                    apply_to_duplicate_texts=True, update_label_counter=True):
@@ -281,7 +283,8 @@ class OrchestratorApi:
         self.data_access.unset_labels(
             workspace_id, category_id, uris, apply_to_duplicate_texts=apply_to_duplicate_texts)
 
-    def get_label_counts(self, workspace_id: str, dataset_name: str, category_id: int, remove_duplicates=False):
+    def get_label_counts(self, workspace_id: str, dataset_name: str, category_id: int, remove_duplicates=False) -> \
+            Mapping[bool, int]:
         """
         Get the number of elements that were labeled for the given category.
         :param workspace_id:
@@ -335,12 +338,13 @@ class OrchestratorApi:
 
     def _delete_category_models(self, workspace_id, category_id):
         workspace = self.orchestrator_state.get_workspace(workspace_id)
-        for idx in range(len(workspace.categories[category_id].iterations)):
-            if workspace.categories[category_id].iterations[idx].model.model_status != ModelStatus.DELETED:
-                self.delete_iteration_model(workspace_id, category_id, idx)
+        if workspace.categories[category_id] is not None:
+            for idx in range(len(workspace.categories[category_id].iterations)):
+                if workspace.categories[category_id].iterations[idx].model.model_status != ModelStatus.DELETED:
+                    self.delete_iteration_model(workspace_id, category_id, idx)
 
     def get_elements_to_label(self, workspace_id: str, category_id: int, count: int, start_index: int = 0) \
-            -> List[TextElement]:
+            -> Tuple[List[TextElement], int]:
         """
         Returns a list of *count* elements recommended for labeling by the active learning module for the latest
         iteration in READY status.
@@ -351,12 +355,12 @@ class OrchestratorApi:
         :return: a list of *count* TextElement objects
         """
         recommended_uris = self.orchestrator_state.get_current_category_recommendations(workspace_id, category_id)
-
+        hit_count = len(recommended_uris)
         if start_index > len(recommended_uris):
             raise Exception(f"exceeded max recommended items. last element index is {len(recommended_uris) - 1}")
         recommended_uris = recommended_uris[start_index:start_index + count]
         dataset_name = self.get_dataset_name(workspace_id)
-        return self.get_text_elements_by_uris(workspace_id, dataset_name, recommended_uris)
+        return self.get_text_elements_by_uris(workspace_id, dataset_name, recommended_uris), hit_count
 
     # Iteration flow
 
@@ -590,11 +594,9 @@ class OrchestratorApi:
 
         iterations = self.orchestrator_state.get_all_iterations(workspace_id, category_id).copy()
 
-
         try:
             iterations_without_errors = [iteration for iteration in iterations
                                          if iteration.status != IterationStatus.ERROR]
-
             label_counts = self.data_access.get_label_counts(workspace_id=workspace_id, dataset_name=dataset_name,
                                                              category_id=category_id, remove_duplicates=True)
             changes_since_last_model = \
@@ -649,8 +651,6 @@ class OrchestratorApi:
 
             logging.exception(f"train_if_recommended failed in iteration {iteration_index}. Model will not be trained")
 
-
-
     def infer(self, workspace_id: str, category_id: int, elements_to_infer: Sequence[TextElement],
               iteration_index: int = None, use_cache: bool = True) -> Sequence[Prediction]:
         """
@@ -666,7 +666,7 @@ class OrchestratorApi:
             return []
 
         iterations = self.get_all_iterations_for_category(workspace_id, category_id)
-        if iteration_index is None:  # use latest ready model
+        if iteration_index is None:  # use the latest ready model
             iteration = [it for it in iterations if it.status == IterationStatus.READY][-1]
         else:
             iteration = iterations[iteration_index]
@@ -748,8 +748,8 @@ class OrchestratorApi:
             elements = self.get_all_unlabeled_text_elements(workspace_id, dataset_name, category_id,
                                                             remove_duplicates=remove_duplicates)
         else:
-            elements = self.data_access.get_text_elements(
-                workspace_id=workspace_id, dataset_name=dataset_name, remove_duplicates=remove_duplicates)["results"]
+            elements = self.data_access.get_text_elements(workspace_id=workspace_id, dataset_name=dataset_name,
+                                                          remove_duplicates=remove_duplicates)["results"]
         predictions = self.infer(workspace_id, category_id, elements)
         elements_with_matching_prediction = [text_element for text_element, prediction in zip(elements, predictions)
                                              if prediction.label == required_label]
@@ -820,32 +820,58 @@ class OrchestratorApi:
                'total': total}
         return res
 
-    def export_workspace_labels(self, workspace_id) -> pd.DataFrame:
+    def export_workspace_labels(self, workspace_id, labeled_only) -> pd.DataFrame:
+        """
+        get all user labels from the workspace as a Dataframe. Each row in the DataFrame is a label for a specific
+        element for a specific category. Column names for the various fields are listed under DisplayFields.
+
+        :param workspace_id:
+        :param labeled_only: only export elements as they were labeled by the user. If set to False, use the
+        TrainingSetSelectionStrategy to determine the exported elements
+        """
         dataset_name = self.get_dataset_name(workspace_id)
         categories = self.get_all_categories(workspace_id)
         list_of_dicts = []
+
         for category_id, category in categories.items():
-            label_count = sum(self.get_label_counts(workspace_id, dataset_name, category_id, False).values())
-            logging.info(f"Total label count in workspace '{workspace_id}' is {label_count}")
-            labeled_elements = self.data_access.get_labeled_text_elements(workspace_id, dataset_name, category_id,
-                                                                          remove_duplicates=False)['results']
-            logging.info(f"Exporting {len(labeled_elements)} unique labeled elements for category id '{category_id}'"
-                         f" from workspace '{workspace_id}'")
+            label_counts = self.get_label_counts(workspace_id, dataset_name, category_id, False)
+            total_count = sum(self.get_label_counts(workspace_id, dataset_name, category_id, False).values())
+
+            if labeled_only or label_counts[LABEL_POSITIVE] == 0:  # if there are no positive elements,
+                # training set selector cannot be used, so we only use the labeled elements
+                logging.info(f"Labeled elements for category {category.name} ({category_id}) in workspace "
+                             f"'{workspace_id}' is {total_count}")
+                text_elements = self.data_access.get_labeled_text_elements(workspace_id, dataset_name, category_id,
+                                                                           remove_duplicates=False)['results']
+            else:
+                train_set_selector = get_training_set_selector(self.data_access,
+                                                               strategy=self.config.training_set_selection_strategy)
+                text_elements = train_set_selector.get_train_set(workspace_id=workspace_id,
+                                                                 train_dataset_name=dataset_name,
+                                                                 category_id=category_id)
+                logging.info(
+                    f"Labeled elements size for category {category.name} ({category_id}) in workspace "
+                    f"'{workspace_id}' is {total_count}, exported elements size is {len(text_elements)}")
+
             list_of_dicts.extend(
                 [{DisplayFields.workspace_id: workspace_id,
                   DisplayFields.category_name: category.name,
-                  DisplayFields.doc_id: le.uri.split('-')[1],  # TODO handle when handling uri/doc_id/element_id
+                  DisplayFields.doc_id: element.uri.split('-')[1],
+                  # TODO handle when handling uri/doc_id/element_id
                   DisplayFields.dataset: dataset_name,
-                  DisplayFields.text: le.text,
-                  DisplayFields.uri: le.uri,
-                  DisplayFields.element_metadata: le.metadata,
-                  DisplayFields.label: le.category_to_label[category_id].label,
+                  DisplayFields.text: element.text,
+                  DisplayFields.uri: element.uri,
+                  DisplayFields.element_metadata: element.metadata,
+                  DisplayFields.label: element.category_to_label[category_id].label,
                   # TODO uncomment when adding metadata
                   # DisplayFields.label_metadata: le.category_to_label[category_id].metadata,
-                  # TODO uncomment when adding support for exporting weak labels
-                  # DisplayFields.label_type: le.category_to_label[category_id].label_type.name
+                  DisplayFields.label_type: element.category_to_label[category_id].label_type.name
                   }
-                 for le in labeled_elements])
+                 for element in text_elements])
+
+        logging.info(
+            f"Exporting a total of {len(list_of_dicts)} elements from workspace '{workspace_id}'")
+
         return pd.DataFrame(list_of_dicts)
 
     def copy_model_dir_for_export(self, workspace_id, category_id, iteration_index):
@@ -916,7 +942,7 @@ class OrchestratorApi:
                      f"'{workspace_id}' after new documents were loaded to dataset '{dataset_name}' "
                      f"using model {iteration_index}")
         # Currently, there is no indication to the user that inference is running on the new documents, and there is no
-        # indication for when this inferences ends. Can be added and reflected in the UI in the future
+        # indication for when this inference ends. Can be added and reflected in the UI in the future
         self.infer(workspace_id, category_id, self.get_all_text_elements(dataset_name), iteration_index)
         logging.info(f"completed inference with the latest model for category id {category_id} in workspace "
                      f"'{workspace_id}' after new documents were loaded to dataset '{dataset_name}',"
